package speechNlu

import (
	"encoding/json"
	"fmt"
	"github.com/gorilla/websocket"
	"io/ioutil"
	"net/http"
	"speech-nlu-parse/global"
	"speech-nlu-parse/model"
	"speech-nlu-parse/pkg/logger"
	"sync"
	"time"
)

type SpeechNlpWs struct {
	NlpWsConn *websocket.Conn
	mutex     sync.Mutex
}

func (this *SpeechNlpWs) SpeechWs() (*websocket.Conn, error) {
	SpeechWs_Url := "wss://dds.dui.ai/dds/v2/test"
	SpeechWs_ProductId := "279629895"
	SpeechWs_ApiKey := "0c74988953dd4ed4bf31955527802cf3"

	//st := time.Now()
	dialer := websocket.Dialer{
		HandshakeTimeout: 30 * time.Second,
	}

	header := http.Header{}
	SpeechUrl := SpeechWs_Url + "?serviceType=websocket&productId=" + SpeechWs_ProductId +
		"&apikey=" + SpeechWs_ApiKey

	fmt.Println(SpeechUrl)

	conn, resp, err := dialer.Dial(SpeechUrl, header)
	if err != nil {
		global.Logger.WithFields(logger.Fields{
			//"RequestId": p.RequestId,
			"resp": readResp(resp),
		}).Errorf("[SpeechWs] Connect error : %v", err.Error())
		//p.Send(ErrorResponse(CodeConnectError, "connect speechWs error"))
		return nil, err
	}
	return conn, nil
}

func (this *SpeechNlpWs) Send(data interface{}) error {
	this.NlpWsConn.WriteJSON(data)
	return nil
}

func (this *SpeechNlpWs) Recv() ([]byte, error) {
	_, msg, err := this.NlpWsConn.ReadMessage()
	return msg, err
}

func readResp(resp *http.Response) string {
	if resp == nil {
		return ""
	}
	b, err := ioutil.ReadAll(resp.Body)
	if err != nil {
		panic(err)
	}
	return fmt.Sprintf("code=%d,body=%s", resp.StatusCode, string(b))
}

func SpeechNlpWsReq(data *model.SemanticReq) *SpeechWsData {
	req := new(SpeechWsData)
	req.Topic = "nlu.input.text"
	req.RecordId = data.Guid
	//sessionId待定
	req.RefText = data.Query
	return req
}

func (this *SpeechNlpWs) ParseSpeechNluData(jsonData []byte, requestId, sessionId string) []byte {
	speechNlpWsResp := model.SpeechWsResp{}
	err := json.Unmarshal(jsonData, &speechNlpWsResp)
	if err != nil {
		global.Logger.WithFields(logger.Fields{
			"data":      map[string]interface{}{"requestBody": string(jsonData)},
			"requestId": requestId,
		}).Error("ParseTNluData Unmarshal error.")
		return replyWithChat(error_reply, "doudi")
	}

	//var jsonByte []byte

	fmt.Println(jsonData)

	return jsonData
}

func (this *SpeechNlpWs) WsSend(body *SpeechWsData) {
	this.Send(body)
}

//type SpeechWsConnection struct {
//	conn  *websocket.Conn
//	mutex sync.Mutex
//}
//
//var SpeechWg = &sync.WaitGroup{}

type SpeechWsData struct {
	Topic     string `json:"topic"`     // 必选
	RecordId  string `json:"recordId"`  // 可选，uuid，请求标识
	SessionId string `json:"sessionId"` // 可选，首轮不带此参数，非首轮必须带此参数
	RefText   string `json:"refText"`   // 必选，请求文本
}

//func SpeechWs(data model.SemanticReq) {
//	da := SpeechNlpWsReq(data)
//
//	st := time.Now()
//	dialer := websocket.Dialer{
//		HandshakeTimeout: 30 * time.Second,
//	}
//
//	header := http.Header{}
//	SpeechUrl := SpeechWs_Url + "?serviceType=websocket&productId=" + SpeechWs_ProductId +
//		"&apikey=" + SpeechWs_ApiKey
//	global.Logger.WithFields(logger.Fields{
//		"Header":    header,
//		"RequestId": data.RequestId,
//		"Url":       SpeechUrl,
//	}).Info("[SpeechWs] dial")
//	conn, resp, err := dialer.Dial(SpeechUrl, header)
//	if err != nil {
//		global.Logger.WithFields(logger.Fields{
//			"RequestId": data.RequestId,
//			"resp":      readResp(resp),
//		}).Errorf("[SpeechWs] Connect error : %v", err.Error())
//		//p.Send(ErrorResponse(CodeConnectError, "connect speechWs error"))
//		return
//	}
//
//	c := SpeechWsConnection{
//		conn:  conn,
//		mutex: sync.Mutex{},
//	}
//	defer c.conn.Close()
//
//	global.Logger.WithFields(logger.Fields{
//		"RequestId": data.RequestId,
//	}).Info("[SpeechWs] Send start")
//
//	SpeechWg.Add(2)
//	go c.read(st, SpeechWg, data)
//	go c.write(da, SpeechWg, data)
//	SpeechWg.Wait()
//}
//
//func (c *SpeechWsConnection) write(v interface{}, wg *sync.WaitGroup, p model.SemanticReq) {
//	defer wg.Done()
//	c.mutex.Lock()
//	if err := c.conn.WriteJSON(v); err != nil {
//		global.Logger.WithFields(logger.Fields{
//			"data":      v,
//			"RequestId": p.RequestId,
//		}).Errorf("[SpeechWs] write error: %v", err.Error())
//	} else {
//		global.Logger.WithFields(logger.Fields{
//			"data":      v,
//			"RequestId": p.RequestId,
//		}).Info("[SpeechWs] write success")
//	}
//	c.mutex.Unlock()
//}
//
//func (c *SpeechWsConnection) read(st time.Time, wg *sync.WaitGroup, p model.SemanticReq) {
//	defer wg.Done()
//
//	// XF使用的是追加上传
//	//var nextPos int64
//	for {
//		var resp = SpeechWsResp{}
//		_, message, err := c.conn.ReadMessage()
//		if err != nil {
//			global.Logger.WithFields(logger.Fields{
//				"RequestId":         p.RequestId,
//				"SpeechWsRequestId": resp.RecordId,
//			}).Errorf("[SpeechWs] 读取message失败: %v", err.Error())
//			//
//			//if websocket.IsCloseError(err, websocket.CloseAbnormalClosure) {
//			//	p.YunXiaoWeiWs()
//			//	continue
//			//}
//
//			//p.Send(ErrorResponse(CodeReadOrParseMsgError, "[SpeechWs] 读取message失败"))
//			break
//		}
//		err = json.Unmarshal(message, &resp)
//		if err != nil {
//			global.Logger.WithFields(logger.Fields{
//				"RequestId":         p.RequestId,
//				"SpeechWsRequestId": resp.RecordId,
//			}).Errorf("[SpeechWs] 解析message失败: %v", err.Error())
//			//p.Send(ErrorResponse(CodeReadOrParseMsgError, "[SpeechWs] 解析message失败"))
//			return
//		}
//		fmt.Println(message)
//		//r := resp.SpeechWsResponse(p.RequestId)
//		//
//		//if resp.Topic == "tts.error" {
//		//	global.Logger.WithFields(logger.Fields{
//		//		"RequestId":         p.RequestId,
//		//		"Code":              resp.Error.ErrId,
//		//		"Message":           resp.Error.Error,
//		//		"Time":              time.Since(st),
//		//		"SpeechWsRequestId": resp.RecordId,
//		//	}).Error("[SpeechWs] 返回message错误")
//		//	r.Status.Code = int32(YXWConvertErrorCode(YXWRespCode(r.Status.Code)))
//		//	p.Send(r)
//		//	return
//		//}
//		//p.Send(r)
//		//
//		//
//		//if resp.Topic == "tts.stream.end" {
//		//	global.Logger.WithFields(logger.Fields{
//		//		"RequestId":         p.RequestId,
//		//		"Topic":             resp.Topic,
//		//		"Time":              time.Since(st).Seconds(),
//		//		"SpeechWsRequestId": resp.RecordId,
//		//	}).Info("[SpeechWs] 读取结束")
//		//
//		//	close(audioChan)
//		//	break
//		//}
//	}
//}
//
//func readResp(resp *http.Response) string {
//	if resp == nil {
//		return ""
//	}
//	b, err := ioutil.ReadAll(resp.Body)
//	if err != nil {
//		panic(err)
//	}
//	return fmt.Sprintf("code=%d,body=%s", resp.StatusCode, string(b))
//}
